// Copyright (C) 2019  Wilko Manger
//
// This Source Code Form is subject to the terms of the Mozilla Public
// License, v. 2.0. If a copy of the MPL was not distributed with this
// file, You can obtain one at http://mozilla.org/MPL/2.0/.

import '../homeserver.dart';
import '../identifier.dart';
import '../my_user.dart';
import 'room.dart';
import '../loadables.dart';
import '../updater.dart';

class Rooms extends Loadables<RoomId, Room> {
  Rooms(
    Iterable<Room> iterable, {
    Context context,
  }) : super(iterable, context: context);

  Rooms.empty({
    Context context,
  }) : super.empty(context: context);

  Room operator [](RoomId id) =>
      firstWhere((s) => s.id == id, orElse: () => null);

  bool containsWithId(RoomId id) => any((r) => r.id == id);

  /// Load more rooms, returning the [Update] where [MyUser] has more rooms.
  @override
  Future<RequestUpdate<Rooms>> load({int count = 20}) async =>
      context.updater.loadRooms(count: count);

  Rooms copyWith({
    Iterable<Room> rooms,
    Context context,
  }) {
    return Rooms(
      rooms ?? this,
      context: context ?? this.context,
    );
  }

  Rooms merge(Rooms other) {
    if (other == null) return this;

    return copyWith(
      rooms: List.of([
        // Merge all rooms that are present in this as well as in other
        ...map(
          (room) => room.merge(
            other.singleWhere(
              (r) => r.equals(room),
              orElse: () => null,
            ),
          ),
        ),
        // All other rooms that were not merged, they're new
        ...other.where(
          (otherRoom) => !any((room) => otherRoom.equals(room)),
        ),
      ], growable: false),
      context: other.context,
    );
  }
}
